# Contributing to the project

## How to contribute

### Reporting Bugs

This section guides you through submitting a bug report for the project. Following these guidelines helps maintainers and the community understand your report, reproduce the behavior, and find related reports.

Before creating bug reports, please check [this list](#before-submitting-a-bug-report) as you might find out that you don't need to create one. When you are creating a bug report, please [include as many details as possible](#how-do-i-submit-a-good-bug-report). Fill out [the required template](.github/ISSUE_TEMPLATE/bug_report.md), the information it asks for helps us resolve issues faster.

#### Before Submitting A Bug Report

* **Check the [documentation](../README.md##-Installation)**. You might be able to find the cause of the problem and fix things yourself. Most importantly, check if you're using the latest version of the software and if the software supports your device. If you're using an older version, please try to reproduce the problem you're having in the latest version of the software.

* **Check the [issue tracker](https://www.github.com/TheModdersDen/BootSoundNX/issues)**. You might be able to find the cause of the problem and fix things yourself. Most importantly, check if the issue has already been reported. If it has and the issue is still open, add a comment to the existing issue instead of opening a new one.

* **Check the [release notes](https://github.com/TheModdersDen/BootSoundNX/releases)**. You might be able to find the cause of the problem and fix things yourself. Most importantly, check if the issue has already been reported. If it has and the issue is still open, add a comment to the existing issue instead of opening a new one.

* **Perform a [cursory search](https://www.github.com/TheModdersDen/BootSoundNX/issues)** to see if the problem has already been reported. If it has and the issue is still open, add a comment to the existing issue instead of opening a new one.

#### How Do I Submit A (Good) Bug Report?

Explain the problem and include additional details to help maintainers reproduce the problem:

* **Use a clear and descriptive title** for the issue to identify the problem.

* **Describe the exact steps which reproduce the problem** in as many details as possible. For example, start by explaining how you started BootSoundNX, e.g. which command exactly you used in the terminal. When listing steps, **don't just say what you did, but explain how you did it**. For example, if you moved the cursor to the end of a line, explain if you used the mouse, or a keyboard shortcut or an IDE action.

* **Provide specific examples to demonstrate the steps**. Include links to files or GitHub projects, or copy/pasteable snippets, which you use in those examples. If you're providing snippets in the issue, use [Markdown code blocks](https://help.github.com/articles/markdown-basics/#multiple-lines).

* **Describe the behavior you observed after following the steps** and point out what exactly is the problem with that behavior.

* **Explain which behavior you expected to see instead and why.**

* **Include screenshots and animated GIFs** which show you following the described steps and clearly demonstrate the problem. You can use [this tool](https://www.cockos.com/licecap/) to record GIFs on macOS and Windows, and [this tool](https://github.com/phw/peek) or [this tool](https://github.com/leafo/gifine) to record GIFs on Linux.

#### How Do I Submit A (Good) Feature Request?

* **Use a clear and descriptive title** for the issue to identify the feature.

* **Provide a step-by-step description of the suggested enhancement** in as many details as possible.

* **Provide specific examples to demonstrate the steps**. Include copy/pasteable snippets which you use in those examples, as [Markdown code blocks](https://help.github.com/articles/markdown-basics/#multiple-lines).

* **Describe the current behavior** and **explain which behavior you expected to see instead** and why.

* **Include screenshots and animated GIFs** which help you demonstrate the steps or point out the part of the project which the suggestion is related to. You can use [this tool](https://www.cockos.com/licecap/) to record GIFs on macOS and Windows, and [this tool](https://github.com/phw/peek) or [this tool](https://github.com/leafo/gifine) to record GIFs on Linux.

* **Explain why this enhancement would be useful** to most users and isn't something that can or should be implemented as a community package.

* **Specify which version of the project you're using.** 

* **Specify the name and version of the Switch you're using.** You can find this information in the System Settings.

* **Specify the version of Atmosphere you're using.** You can find this information in the System Settings.

* **Specify the version of Hekate you're using.** You can find this information in the System Settings.

#### Submitting a Pull Request

* **Fork the repository** and clone it to your local machine.

* **Create a new branch** for your changes.

* **Make your changes** to the source code.

* **Test your changes** to the source code.

* **Commit your changes** to the new branch.

* **Push your changes** to your fork.

* **Submit a pull request** to the original repository.

* **Wait for a response** from the maintainers.

* **Make changes to the pull request** if the maintainers recommend them.

* **Celebrate** your success if and or when your pull request is merged!

#### Code of Conduct

This project and everyone participating in it is governed by the [Contributor Covenant Code of Conduct](CODE_OF_CONDUCT.md). By participating, you are expected to uphold this code. Please report unacceptable behavior to [TheModdersDen](https://www.github.com/TheModdersDen).

#### License

